@@ -66,6 +66,13 @@ typedef enum
     ENGINE_FOCUS_CATEGORY_HAS_ID
 } EngineFocusCategory;
 
+typedef enum
+{
+    ENGINE_SURROUNDING_TEXT_CATEGORY_NONE = 0,
+    ENGINE_SURROUNDING_TEXT_CATEGORY_NOT_ACTIVE,
+    ENGINE_SURROUNDING_TEXT_CATEGORY_HAS_ACTIVE
+} EngineSurroundingTextCategory;
+
 GType            bus_ibus_impl_get_type             (void);
 
 /**
@@ -91,5 +98,7 @@ BusInputContext *bus_ibus_impl_get_focused_input_context
                                                     (BusIBusImpl        *ibus);
 GHashTable      *bus_ibus_impl_get_engine_focus_id_table
                                                     (BusIBusImpl        *ibus);
+GHashTable      *bus_ibus_impl_get_engine_active_surrounding_text_table
+                                                    (BusIBusImpl        *ibus);
 G_END_DECLS
 #endif
