@@ -2,8 +2,8 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2015-2022 Takao Fujiwara <takao.fujiwara1@gmail.com>
- * Copyright (C) 2008-2022 Red Hat, Inc.
+ * Copyright (C) 2015-2023 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2008-2023 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -213,7 +213,7 @@ static gboolean _slave_delete_surrounding_cb
                                              gint                offset_from_cursor,
                                              guint               nchars,
                                              IBusIMContext      *context);
-static void     _request_surrounding_text   (IBusIMContext      *context);
+static gboolean _request_surrounding_text   (IBusIMContext      *context);
 static void     _create_fake_input_context  (void);
 static gboolean _set_content_type           (IBusIMContext      *context);
 
@@ -381,8 +381,9 @@ typedef struct {
 } ProcessKeyEventData;
 
 typedef struct {
-    GMainLoop *loop;
-    gboolean    retval;
+    int       count;
+    guint     count_cb_id;
+    gboolean  retval;
 } ProcessKeyEventReplyData;
 
 static void
@@ -452,7 +453,23 @@ _process_key_event_reply_done (GObject      *object,
     }
     g_return_if_fail (data);
     data->retval = retval;
-    g_main_loop_quit (data->loop);
+    data->count = 0;
+    g_source_remove (data->count_cb_id);
+}
+
+static gboolean
+_process_key_event_count_cb (gpointer user_data)
+{
+    ProcessKeyEventReplyData *data = (ProcessKeyEventReplyData *)user_data;
+    g_return_val_if_fail (data, G_SOURCE_REMOVE);
+    if (!data->count)
+        return G_SOURCE_REMOVE;
+    /* Wait for about 10 secs. */
+    if (data->count++ == 10000) {
+        data->count = 0;
+        return G_SOURCE_REMOVE;
+    }
+    return G_SOURCE_CONTINUE;
 }
 
 static gboolean
@@ -495,10 +512,10 @@ _process_key_event (IBusInputContext *context,
         break;
     }
     case 2: {
-        GMainLoop *loop = g_main_loop_new (NULL, TRUE);
+        GSource *source = g_timeout_source_new (1);
         ProcessKeyEventReplyData *data = NULL;
 
-        if (loop)
+        if (source)
             data = g_slice_new0 (ProcessKeyEventReplyData);
         if (!data) {
             g_warning ("Cannot wait for the reply of the process key event.");
@@ -506,11 +523,14 @@ _process_key_event (IBusInputContext *context,
                                                            keyval,
                                                            keycode - 8,
                                                            state);
-            if (loop)
-                g_main_loop_quit (loop);
+            if (source)
+                g_source_destroy (source);
             break;
         }
-        data->loop = loop;
+        data->count = 1;
+        g_source_attach (source, NULL);
+        g_source_unref (source);
+        data->count_cb_id = g_source_get_id (source);
         ibus_input_context_process_key_event_async (context,
             keyval,
             keycode - 8,
@@ -519,7 +539,14 @@ _process_key_event (IBusInputContext *context,
             NULL,
             _process_key_event_reply_done,
             data);
-        g_main_loop_run (loop);
+        g_source_set_callback (source, _process_key_event_count_cb, data, NULL);
+        while (data->count)
+            g_main_context_iteration (NULL, TRUE);
+        if (source->ref_count > 0) {
+            /* g_source_get_id() could causes a SEGV */
+            g_info ("Broken GSource.ref_count and maybe a timing issue in %p.",
+                    source);
+        }
         retval = data->retval;
         g_slice_free (ProcessKeyEventReplyData, data);
         break;
@@ -564,26 +591,19 @@ _process_key_event (IBusInputContext *context,
  * context->caps has IBUS_CAP_SURROUNDING_TEXT and the current IBus
  * engine needs surrounding-text.
  */
-static void
+static gboolean
 _request_surrounding_text (IBusIMContext *context)
 {
+    gboolean return_value = TRUE;
     if (context &&
         (context->caps & IBUS_CAP_SURROUNDING_TEXT) != 0 &&
         context->ibuscontext != NULL &&
         ibus_input_context_needs_surrounding_text (context->ibuscontext)) {
-        gboolean return_value;
         IDEBUG ("requesting surrounding text");
         g_signal_emit (context, _signal_retrieve_surrounding_id, 0,
                        &return_value);
-        if (!return_value) {
-            /* #2054 firefox::IMContextWrapper::GetCurrentParagraph() could
-             * fail with the first typing on firefox but it succeeds with
-             * the second typing.
-             */
-            g_warning ("%s has no capability of surrounding-text feature",
-                       g_get_prgname ());
-        }
     }
+    return return_value;
 }
 
 static gboolean
@@ -984,8 +1004,8 @@ ibus_im_context_init (GObject *obj)
 #else
     ibusimcontext->caps = IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_FOCUS;
 #endif
-    if (_use_sync_mode != 1)
-        ibusimcontext->caps |= IBUS_CAP_SYNC_PROCESS_KEY;
+    if (_use_sync_mode == 1)
+        ibusimcontext->caps |= IBUS_CAP_SYNC_PROCESS_KEY_V2;
 
     ibusimcontext->events_queue = g_queue_new ();
 
@@ -1328,7 +1348,7 @@ ibus_im_context_reset (GtkIMContext *context)
          * IBus uses button-press-event instead until GTK is fixed.
          * https://gitlab.gnome.org/GNOME/gtk/issues/1534
          */
-        if (_use_sync_mode == 1)
+        if (_use_sync_mode != 0)
             ibus_im_context_clear_preedit_text (ibusimcontext);
         ibus_input_context_reset (ibusimcontext->ibuscontext);
     }
@@ -1443,7 +1463,7 @@ ibus_im_context_set_client_window (GtkIMContext *context,
 
     if (ibusimcontext->client_window) {
 #if !GTK_CHECK_VERSION (3, 98, 4)
-        if (ibusimcontext->use_button_press_event && _use_sync_mode != 1)
+        if (ibusimcontext->use_button_press_event && _use_sync_mode == 0)
             _connect_button_press_event (ibusimcontext, FALSE);
 #endif
         g_object_unref (ibusimcontext->client_window);
@@ -1453,7 +1473,7 @@ ibus_im_context_set_client_window (GtkIMContext *context,
     if (client != NULL) {
         ibusimcontext->client_window = g_object_ref (client);
 #if !GTK_CHECK_VERSION (3, 98, 4)
-        if (!ibusimcontext->use_button_press_event && _use_sync_mode != 1)
+        if (!ibusimcontext->use_button_press_event && _use_sync_mode == 0)
             _connect_button_press_event (ibusimcontext, TRUE);
 #endif
     }
@@ -2075,7 +2095,7 @@ _ibus_context_update_preedit_text_cb (IBusInputContext  *ibuscontext,
 #if !GTK_CHECK_VERSION (3, 98, 4)
     if (!ibusimcontext->use_button_press_event &&
         mode == IBUS_ENGINE_PREEDIT_COMMIT &&
-        _use_sync_mode != 1) {
+        _use_sync_mode == 0) {
         if (ibusimcontext->client_window) {
             _connect_button_press_event (ibusimcontext, TRUE);
         }
@@ -2175,6 +2195,35 @@ _ibus_context_hide_preedit_text_cb (IBusInputContext *ibuscontext,
     g_signal_emit (ibusimcontext, _signal_preedit_end_id, 0);
 }
 
+static void
+_ibus_warn_no_support_surrounding_text (IBusIMContext *context)
+{
+    /* Engines can disable the surrounding text feature with
+     * the updated capabilities.
+     */
+    if (context->caps & IBUS_CAP_SURROUNDING_TEXT) {
+        context->caps &= ~IBUS_CAP_SURROUNDING_TEXT;
+        ibus_input_context_set_capabilities (context->ibuscontext,
+                                             context->caps);
+    }
+    g_warning ("%s has no capability of surrounding-text feature",
+               g_get_prgname ());
+}
+
+static void
+_ibus_context_require_surrounding_text_cb (IBusInputContext *ibuscontext,
+                                           IBusIMContext    *ibusimcontext)
+{
+    IDEBUG ("%s", __FUNCTION__);
+    g_assert (ibusimcontext->ibuscontext == ibuscontext);
+    if (!_request_surrounding_text (ibusimcontext))
+        _ibus_warn_no_support_surrounding_text (ibusimcontext);
+    g_signal_handlers_disconnect_by_func (
+            ibusimcontext->ibuscontext,
+            G_CALLBACK (_ibus_context_require_surrounding_text_cb),
+            ibusimcontext);
+}
+
 static void
 _ibus_context_destroy_cb (IBusInputContext *ibuscontext,
                           IBusIMContext    *ibusimcontext)
@@ -2214,6 +2263,7 @@ _create_input_context_done (IBusBus       *bus,
         g_error_free (error);
     }
     else {
+        gboolean requested_surrounding_text = FALSE;
         ibus_input_context_set_client_commit_preedit (context, TRUE);
         ibusimcontext->ibuscontext = context;
 
@@ -2245,7 +2295,8 @@ _create_input_context_done (IBusBus       *bus,
                           G_CALLBACK (_ibus_context_destroy_cb),
                           ibusimcontext);
 
-        ibus_input_context_set_capabilities (ibusimcontext->ibuscontext, ibusimcontext->caps);
+        ibus_input_context_set_capabilities (ibusimcontext->ibuscontext,
+                                             ibusimcontext->caps);
 
         if (ibusimcontext->has_focus) {
             /* The time order is _create_input_context() ->
@@ -2257,6 +2308,19 @@ _create_input_context_done (IBusBus       *bus,
 
             ibus_input_context_focus_in (ibusimcontext->ibuscontext);
             _set_cursor_location_internal (ibusimcontext);
+            if (ibus_input_context_needs_surrounding_text (
+                        ibusimcontext->ibuscontext)) {
+                if (!_request_surrounding_text (ibusimcontext))
+                    _ibus_warn_no_support_surrounding_text (ibusimcontext);
+                requested_surrounding_text = TRUE;
+            }
+        }
+        if (!requested_surrounding_text) {
+            g_signal_connect (
+                    ibusimcontext->ibuscontext,
+                    "require-surrounding-text",
+                    G_CALLBACK (_ibus_context_require_surrounding_text_cb),
+                    ibusimcontext);
         }
 
         if (!g_queue_is_empty (ibusimcontext->events_queue)) {
@@ -2426,8 +2490,8 @@ _create_fake_input_context_done (IBusBus       *bus,
                       NULL);
 
     guint32 caps = IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_FOCUS | IBUS_CAP_SURROUNDING_TEXT;
-    if (_use_sync_mode != 1)
-        caps |= IBUS_CAP_SYNC_PROCESS_KEY;
+    if (_use_sync_mode == 1)
+        caps |= IBUS_CAP_SYNC_PROCESS_KEY_V2;
     ibus_input_context_set_capabilities (_fake_context, caps);
 
     /* focus in/out the fake context */
