@@ -2,8 +2,8 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2015 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2015-2018 Takao Fujiwara <takao.fujiwara1@gmail.com>
- * Copyright (C) 2008-2018 Red Hat, Inc.
+ * Copyright (C) 2015-2023 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2008-2019 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include "ibusinternal.h"
 
 #define IBUS_SERVICE_GET_PRIVATE(o)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((o), IBUS_TYPE_SERVICE, IBusServicePrivate))
+   ((IBusServicePrivate *)ibus_service_get_instance_private (o))
 
 enum {
     LAST_SIGNAL
@@ -133,6 +133,13 @@ static const GDBusInterfaceVTable ibus_service_interface_vtable = {
 };
 
 static IBusObjectClass *ibus_service_parent_class = NULL;
+static gint ibus_service_private_offset;
+
+static inline gpointer
+ibus_service_get_instance_private (IBusService *self)
+{
+    return (G_STRUCT_MEMBER_P (self, ibus_service_private_offset));
+}
 
 GType
 ibus_service_get_type (void)
@@ -156,6 +163,9 @@ ibus_service_get_type (void)
                                        "IBusService",
                                        &type_info,
                                        0);
+        ibus_service_private_offset =
+                g_type_add_instance_private (type,
+                                             sizeof (IBusServicePrivate));
     }
 
     return type;
@@ -198,12 +208,20 @@ ibus_service_class_init (IBusServiceClass *class)
     GObjectClass *gobject_class = G_OBJECT_CLASS (class);
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (class);
 
-    ibus_service_parent_class = IBUS_OBJECT_CLASS (g_type_class_peek_parent (class));
+    ibus_service_parent_class =
+            IBUS_OBJECT_CLASS (g_type_class_peek_parent (class));
+    if (ibus_service_private_offset) {
+        g_type_class_adjust_private_offset (class,
+                                            &ibus_service_private_offset);
+    }
 
     gobject_class->constructed  = ibus_service_constructed;
-    gobject_class->set_property = (GObjectSetPropertyFunc) ibus_service_set_property;
-    gobject_class->get_property = (GObjectGetPropertyFunc) ibus_service_get_property;
-    ibus_object_class->destroy  = (IBusObjectDestroyFunc) ibus_service_destroy;
+    gobject_class->set_property =
+            (GObjectSetPropertyFunc) ibus_service_set_property;
+    gobject_class->get_property =
+            (GObjectGetPropertyFunc) ibus_service_get_property;
+    ibus_object_class->destroy  =
+            (IBusObjectDestroyFunc) ibus_service_destroy;
 
     /* virtual functions */
     class->service_method_call = ibus_service_service_method_call;
@@ -252,8 +270,6 @@ ibus_service_class_init (IBusServiceClass *class)
                         G_PARAM_STATIC_NICK |
                         G_PARAM_STATIC_BLURB)
                     );
-
-    g_type_class_add_private (class, sizeof (IBusServicePrivate));
 }
 
 static void
@@ -354,8 +370,9 @@ ibus_service_service_method_call (IBusService           *service,
     g_dbus_method_invocation_return_error (invocation,
                                            G_DBUS_ERROR,
                                            G_DBUS_ERROR_UNKNOWN_METHOD,
-                                           "%s::%s", interface_name, method_name);
-    return;
+                                           "%s::%s",
+                                           interface_name, method_name);
+    g_return_if_reached ();
 }
 
 static GVariant *
@@ -367,7 +384,15 @@ ibus_service_service_get_property (IBusService     *service,
                                   const gchar      *property_name,
                                   GError          **error)
 {
-    return NULL;
+    if (error) {
+        *error = NULL;
+        g_set_error (error,
+                     G_DBUS_ERROR,
+                     G_DBUS_ERROR_FAILED,
+                     "service_get_property received an unknown property: %s",
+                     property_name ? property_name : "(null)");
+    }
+    g_return_val_if_reached (NULL);
 }
 
 static gboolean
@@ -380,7 +405,15 @@ ibus_service_service_set_property (IBusService     *service,
                                   GVariant         *value,
                                   GError          **error)
 {
-    return FALSE;
+    if (error) {
+        *error = NULL;
+        g_set_error (error,
+                     G_DBUS_ERROR,
+                     G_DBUS_ERROR_FAILED,
+                     "service_set_property received an unknown property: %s",
+                     property_name ? property_name : "(null)");
+    }
+    g_return_val_if_reached (FALSE);
 }
 
 static void
