@@ -2,7 +2,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2015-2021 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2015-2023 Takao Fujiwara <takao.fujiwara1@gmail.com>
  * Copyright (C) 2008-2018 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -24,6 +24,7 @@
 #include <config.h>
 #endif
 
+#include "ibusresources.h"
 #include "ibusshare.h"
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -313,6 +314,7 @@ ibus_init (void)
     IBUS_TYPE_OBSERVED_PATH;
     IBUS_TYPE_REGISTRY;
     IBUS_TYPE_X_EVENT;
+    _ibus_register_resource ();
 }
 
 static GMainLoop *main_loop = NULL;
